//
//  AppDelegate.swift
//  URLServiceRouterDemo
//
//  Created by huanyu.li on 2021/8/4.
//

import UIKit
import URLServiceRouter

@main
class AppDelegate: UIResponder, UIApplicationDelegate {
    var window: UIWindow?

    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey: Any]?) -> Bool {
        congifURLServiceRouter()
        
        window = UIWindow.init(frame: UIScreen.main.bounds)
        window?.backgroundColor = .white
        window?.makeKeyAndVisible()
        
        window?.rootViewController = UINavigationController.init(rootViewController: ViewController())
        return true
    }
    
    func congifURLServiceRouter() -> Void {
        URLServiceRouter.shared.config(delegate: URLServiceRouterDelegate())
        registerServices()
        registerNodes()
    }
    
    func registerServices() {
        URLServiceRouter.shared.register(service: URLOwnerInfoService())
        URLServiceRouter.shared.register(service: InputPageService())
    }
    
    func registerNodes() {
        URLServiceRouter.shared.registerNode(from: "https", parsers:[URLServiceRedirectTestHostParser()])
        
        do {
            let parser = URLServiceNoramlParser(parserType: .post, parseBlock: { (nodeParser, request, decision) in
                decision.complete(nodeParser ,"user://info")
            })
             
            URLServiceRouter.shared.registerNode(from: "https://www.realword.com/owner/info", parsers: [parser]);
        }

        do {
            let preParser = URLServiceNoramlParser(parserType: .pre, parseBlock: { (nodeParser, request, decision) in
                var nodeNames = request.nodeNames
                if let first = nodeNames.first, first.isPureInt {
                    request.merge(params: ["id": nodeNames.remove(at: 0)], from: nodeParser)
                    request.replace(nodeNames: nodeNames, from: nodeParser)
                }
                decision.next()
            })
            
            let postParser = URLServiceNoramlParser(parserType: .post, parseBlock: { (nodeParser, request, decision) in
                decision.next()
            })
             
            URLServiceRouter.shared.registerNode(from: "https://www.realword.com/owner/", parsers:[preParser, postParser]);
        }
        
        URLServiceRouter.shared.registerNode(from: "https://www.realword.com/company/work");
    }
}

